unit DemoU;
{
QueryInfo version 0.9 preview release demo program by Justin Turberville.
Make sure that all components used here are first compiled into your DCL file.
Assign QIArchiver.ArchivePath if its tables are not in the program directory.
Check out how to get the joins working (I think) in procedure AssignQuery.
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBTables, DB, MemoDlg, qiQuery, qiLookup, qiOrder, qiGroup, qiFilter,
  qiSelect, qiFormat, StdCtrls, qiInfo, TablePrn, qiArchiv;

type
  TDemoForm = class(TForm)
    QueryInfo: TQueryInfo;
    Label1: TLabel;
    Label2: TLabel;
    Button1: TButton;
    Formatter: TQIFormatter;
    LookupDlg: TQILookupDialog;
    EditDlg: TMemoDialog;
    Customers: TTable;
    Orders: TTable;
    CustomersCustNo: TFloatField;
    CustomersCompany: TStringField;
    CustomersAddr1: TStringField;
    CustomersCity: TStringField;
    OrdersOrderNo: TFloatField;
    OrdersCustNo: TFloatField;
    OrdersSaleDate: TDateTimeField;
    OrdersPaymentMethod: TStringField;
    OrdersTaxRate: TFloatField;
    OrdersAmountPaid: TCurrencyField;
    Employees: TTable;
    Items: TTable;
    OrdersEmpNo: TIntegerField;
    Parts: TTable;
    Vendors: TTable;
    EmployeesEmpNo: TIntegerField;
    EmployeesLastName: TStringField;
    EmployeesFirstName: TStringField;
    VendorsVendorNo: TFloatField;
    VendorsVendorName: TStringField;
    VendorsCity: TStringField;
    VendorsPreferred: TBooleanField;
    ItemsOrderNo: TFloatField;
    ItemsItemNo: TFloatField;
    ItemsPartNo: TFloatField;
    ItemsQty: TIntegerField;
    ItemsDiscount: TFloatField;
    PartsPartNo: TFloatField;
    PartsVendorNo: TFloatField;
    PartsDescription: TStringField;
    PartsListPrice: TCurrencyField;
    TablePrinter: TTablePrinter;
    QueryWdw: TQIQueryWindow;
    Archiver: TQIArchiver;
    SelectDlg: TQISelectDialog;
    GroupDlg: TQIGroupDialog;
    OrderDlg: TQIOrderDialog;
    FilterDlg: TQIFilterDialog;
    procedure Button1Click(Sender: TObject);
    procedure QueryWdwSelect(Sender: TObject);
    procedure QueryWdwFilter(Sender: TObject);
    procedure QueryWdwOrder(Sender: TObject);
    procedure QueryWdwEditSQL(Sender: TObject);
    procedure SelectDlgGrouping(Sender: TObject);
    procedure QueryWdwLookup(Sender: TObject);
    procedure QueryWdwFindNext(Sender: TObject);
    procedure QueryWdwPrint(Sender: TObject);
    procedure QueryWdwLoad(Sender: TObject);
    procedure QueryWdwSave(Sender: TObject);
  private
    procedure AssignQuery;
  public
  end;

var
  DemoForm: TDemoForm;

implementation

{$R *.DFM}

procedure TDemoForm.Button1Click(Sender: TObject);
begin
  QueryWdw.Show;
  AssignQuery;
end;

procedure TDemoForm.QueryWdwSelect(Sender: TObject);
begin
  if SelectDlg.Execute then AssignQuery;
end;

procedure TDemoForm.SelectDlgGrouping(Sender: TObject);
begin
  if GroupDlg.Execute then AssignQuery;
end;

procedure TDemoForm.QueryWdwFilter(Sender: TObject);
begin
  if FilterDlg.Execute then AssignQuery;
end;

procedure TDemoForm.QueryWdwOrder(Sender: TObject);
begin
  if OrderDlg.Execute then AssignQuery;
end;

procedure TDemoForm.QueryWdwLoad(Sender: TObject);
begin
  if Archiver.Load then AssignQuery;
end;

procedure TDemoForm.QueryWdwSave(Sender: TObject);
begin
  Archiver.Store('', QueryWdw.DisplayScript);
end;

procedure TDemoForm.QueryWdwEditSQL(Sender: TObject);
begin
  EditDlg.Execute(QueryWdw.SQLScript);
end;

procedure TDemoForm.QueryWdwLookup(Sender: TObject);
begin
  if not QueryWdw.QuerySource.DataSet.Active then Exit;
  if LookupDlg.Execute then
    with QueryInfo.Fields.Lookup do
      LookupDlg.FindFirstValue(TQuery(QueryWdw.QuerySource.DataSet),
        Field.SQLName, Value);
end;

procedure TDemoForm.QueryWdwFindNext(Sender: TObject);
begin
  if not QueryWdw.QuerySource.DataSet.Active then Exit;
  with QueryInfo.Fields.Lookup do
    LookupDlg.FindNextValue(TQuery(QueryWdw.QuerySource.DataSet),
      Field.SQLName, Value);
end;

procedure TDemoForm.QueryWdwPrint(Sender: TObject);
var
  i: Integer;
  s: string;
begin
  with TablePrinter, QueryWdw do begin
    DataSource := QuerySource;
    s := '';
    for i := 0 to DisplayScript.Count-1 do
      s := s + DisplayScript[i] + '.  ';
    SubHeader := s;
    Execute;
  end;
end;

{ join links:   Vendors <-> Parts <-> Items <-> Orders <-> Customers
                                                   ^-> Employees   }
procedure TDemoForm.AssignQuery;
var
  i: Integer;
  item, part, vendor, order, cust, empl: TTableItem;
begin
  with QueryInfo do begin   {ensure bridging joins made where required -}
    for i := 0 to Tables.Count-1 do
      Tables[i].Master := False;
    item   := Tables.ByTableRef(Items);
    part   := Tables.ByTableRef(Parts);
    vendor := Tables.ByTableRef(Vendors);
    order  := Tables.ByTableRef(Orders);
    cust   := Tables.ByTableRef(Customers);
    empl   := Tables.ByTableRef(Employees);
    part.Master :=                                   {a table item with its    }
      (TableUsed(vendor) and TableUsed(item))  or    {master property set will }
      (TableUsed(vendor) and TableUsed(order)) or    {always be included in    }
      (TableUsed(vendor) and TableUsed(cust))  or    {the FROM clause and its  }
      (TableUsed(vendor) and TableUsed(empl));       {common joins defined by  }
    item.Master :=                                   {QIFormatter in the script}
      (TableUsed(part)   and TableUsed(order)) or
      (TableUsed(part)   and TableUsed(cust))  or
      (TableUsed(part)   and TableUsed(empl))  or
      (TableUsed(vendor) and TableUsed(order)) or
      (TableUsed(vendor) and TableUsed(cust))  or
      (TableUsed(vendor) and TableUsed(empl));
    order.Master :=
      (TableUsed(cust) and TableUsed(item))   or
      (TableUsed(cust) and TableUsed(part))   or
      (TableUsed(cust) and TableUsed(vendor)) or
      (TableUsed(cust) and TableUsed(empl))   or
      (TableUsed(empl) and TableUsed(item))   or
      (TableUsed(empl) and TableUsed(part))   or
      (TableUsed(empl) and TableUsed(vendor));             {phew!}
  end;
  Formatter.BuildUserQuery(QueryWdw.DisplayScript);
  Formatter.BuildSQLQuery(QueryWdw.SQLScript);
end;

end.
